package com.jiuqi.util;

import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.text.DecimalFormat;

public class Format {
   public static char fillChar = ' ';
   private static final String EOL = System.getProperty("line.separator");
   private static PrintWriter handle;
   private static char alignment;
   private static boolean separator;
   private static char base;
   private static boolean scientific;
   private static boolean zeroFill;
   private static int width;
   private static int decimals;
   private static String pattern;

   static {
      handle = new PrintWriter(new OutputStreamWriter(System.out));
   }

   private Format() {
   }

   public static String format(byte value, String fd) {
      return formatInteger((long)value, fd, 8);
   }

   public static String format(char value, String fd) {
      return formatInteger((long)value, fd, 16);
   }

   public static String format(double value, String fd) {
      extractAttributes(fd);
      String s1;
      if (base == 'B') {
         s1 = Long.toBinaryString(Double.doubleToLongBits(value));
         s1 = repeat(64, '0') + s1;
         s1 = s1.substring(s1.length() - 64);
      } else if (base == 'X') {
         s1 = Long.toHexString(Double.doubleToLongBits(value));
         s1 = repeat(16, '0') + s1;
         s1 = s1.substring(s1.length() - 16);
      } else {
         pattern = decimals != -1 ? "." + repeat(decimals, '0') : ".#";
         if (scientific) {
            pattern = "0" + pattern + "E0";
         } else {
            pattern = separator ? "#,##0" + pattern : "0" + pattern;
         }

         s1 = (new DecimalFormat(pattern)).format(value);
      }

      return size(s1);
   }

   public static String format(float value, String fd) {
      extractAttributes(fd);
      String s1;
      if (base == 'B') {
         s1 = Integer.toBinaryString(Float.floatToIntBits(value));
         s1 = repeat(32, '0') + s1;
         s1 = s1.substring(s1.length() - 32);
      } else if (base == 'X') {
         s1 = Integer.toHexString(Float.floatToIntBits(value));
         s1 = repeat(8, '0') + s1;
         s1 = s1.substring(s1.length() - 8);
      } else {
         pattern = decimals != -1 ? "." + repeat(decimals, '0') : ".#";
         if (scientific) {
            pattern = "0" + pattern + "E0";
         } else {
            pattern = separator ? "#,##0" + pattern : "0" + pattern;
         }

         s1 = (new DecimalFormat(pattern)).format((double)value);
      }

      return size(s1);
   }

   public static String format(int value, String fd) {
      return formatInteger((long)value, fd, 32);
   }

   public static String format(long value, String fd) {
      return formatInteger(value, fd, 64);
   }

   public static String format(short value, String fd) {
      return formatInteger((long)value, fd, 16);
   }

   private static String formatInteger(long l, String s, int i) {
      extractAttributes(s);
      String s1;
      if (base == 'B') {
         s1 = Long.toBinaryString(l);
         s1 = repeat(64, '0') + s1;
         s1 = s1.substring(s1.length() - i);
      } else if (base == 'X') {
         s1 = Long.toHexString(l);
         s1 = repeat(16, '0') + s1;
         s1 = s1.substring(s1.length() - i / 4);
      } else if (separator) {
         s1 = (new DecimalFormat("#,###")).format(l);
      } else {
         s1 = String.valueOf(l);
         if (zeroFill) {
            s1 = repeat(width - s1.length(), '0') + s1;
         }
      }

      return size(s1);
   }

   private static void extractAttributes(String s) {
      s = s.toUpperCase();
      alignment = 'R';
      separator = false;
      base = 'D';
      scientific = false;
      zeroFill = false;
      width = -1;
      decimals = -1;
      int i = s.indexOf(76, 0);
      if (i > -1) {
         alignment = 'L';
         s = s.substring(0, i) + s.substring(i + 1);
      }

      i = s.indexOf(67, 0);
      if (i > -1) {
         alignment = 'C';
         s = s.substring(0, i) + s.substring(i + 1);
      }

      i = s.indexOf(44, 0);
      if (i > -1) {
         separator = true;
         pattern = pattern + ",###";
         s = s.substring(0, i) + s.substring(i + 1);
      }

      i = s.indexOf(88, 0);
      if (i > -1) {
         base = 'X';
         s = s.substring(0, i) + s.substring(i + 1);
      } else {
         i = s.indexOf(66, 0);
         if (i > -1) {
            base = 'B';
            s = s.substring(0, i) + s.substring(i + 1);
         }
      }

      i = s.indexOf(83, 0);
      if (i > -1) {
         scientific = true;
         s = s.substring(0, i) + s.substring(i + 1);
      }

      i = s.indexOf(90, 0);
      if (i > -1) {
         zeroFill = true;
         s = s.substring(0, i) + s.substring(i + 1);
      }

      i = s.indexOf(46, 0);
      if (i > -1) {
         decimals = Integer.parseInt(s.substring(i + 1));
         s = s.substring(0, i);
      }

      if (s.length() > 0) {
         width = Integer.parseInt(s);
      }

   }

   public static void print(byte value, String fd) {
      handle.print(format(value, fd));
      handle.flush();
   }

   public static void print(char value, String fd) {
      handle.print(format(value, fd));
      handle.flush();
   }

   public static void print(double value, String fd) {
      handle.print(format(value, fd));
      handle.flush();
   }

   public static void print(float value, String fd) {
      handle.print(format(value, fd));
      handle.flush();
   }

   public static void print(int value, String fd) {
      handle.print(format(value, fd));
      handle.flush();
   }

   public static void print(long value, String fd) {
      handle.print(format(value, fd));
      handle.flush();
   }

   public static void print(short value, String fd) {
      handle.print(format(value, fd));
      handle.flush();
   }

   public static void println(byte value, String fd) {
      print(value, fd);
      handle.print(EOL);
      handle.flush();
   }

   public static void println(char value, String fd) {
      print(value, fd);
      handle.print(EOL);
      handle.flush();
   }

   public static void println(double value, String fd) {
      print(value, fd);
      handle.print(EOL);
      handle.flush();
   }

   public static void println(float value, String fd) {
      print(value, fd);
      handle.print(EOL);
      handle.flush();
   }

   public static void println(int value, String fd) {
      print(value, fd);
      handle.print(EOL);
      handle.flush();
   }

   public static void println(long value, String fd) {
      print(value, fd);
      handle.print(EOL);
      handle.flush();
   }

   public static void println(short value, String fd) {
      print(value, fd);
      handle.print(EOL);
      handle.flush();
   }

   private static String size(String s) {
      int i = width - s.length();
      if (alignment == 'R') {
         return repeat(i, fillChar) + s;
      } else {
         return alignment == 'L' ? s + repeat(i, fillChar) : repeat(i / 2, fillChar) + s + repeat(i / 2 + i % 2, fillChar);
      }
   }

   public static String repeat(int times, char c) {
      String s = "";

      for(int i = 0; i < times; ++i) {
         s = s + c;
      }

      return s;
   }
}
